@@ -1,7 +1,5 @@
 from __future__ import absolute_import, division, unicode_literals
 
-import param as _param
-
 from . import layout # noqa
 from . import links # noqa
 from . import pane # noqa
@@ -9,15 +7,15 @@ from . import param # noqa
 from . import pipeline # noqa
 from . import widgets # noqa
 
-from .config import config, panel_extension as extension # noqa
-from .depends import depends # noqa
+from .config import config, panel_extension as extension, __version__ # noqa
+from .depends import bind, depends # noqa
 from .interact import interact # noqa
 from .io import ipywidget, serve, state # noqa
-from .layout import Row, Column, WidgetBox, Tabs, Spacer, GridSpec, GridBox # noqa
+from .layout import ( # noqa
+    Accordion, Card, Row, Column, WidgetBox, Tabs, Spacer, 
+    GridSpec, GridBox
+)
 from .pane import panel, Pane # noqa
 from .param import Param # noqa
 from .template import Template # noqa
-
-__version__ = str(_param.version.Version(
-    fpath=__file__, archive_commit="$Format:%h$", reponame="panel"))
-
+from .widgets import indicators # noqa
