@@ -5,6 +5,11 @@ defined as pairs of Python classes and TypeScript models defined in .ts
 files.
 """
 
+from .ipywidget import IPyWidget # noqa
+from .layout import Card # noqa
+from .location import Location # noqa
 from .markup import JSON, HTML # noqa
 from .state import State # noqa
-from .widgets import Audio, FileDownload, Player, Progress, Video, VideoStream # noqa
+from .widgets import ( # noqa
+    Audio, FileDownload, Player, Progress, SingleSelect, Video, VideoStream
+)
