@@ -63,5 +63,5 @@ class Ace(PaneBase):
         self._models[root.ref['id']] = (model, parent)
         return model
 
-    def _update(self, model):
+    def _update(self, ref=None, model=None):
         model.code = self.object if self.object else ''
