@@ -391,6 +391,9 @@ class Param(PaneBase):
 
         kwargs = {k: v for k, v in kw.items() if k in widget_class.param}
 
+        if isinstance(widget_class, type) and issubclass(widget_class, Button):
+            kwargs.pop('value', None)
+
         if isinstance(widget_class, Widget):
             widget = widget_class
         else:
@@ -416,6 +419,8 @@ class Param(PaneBase):
             def action(change):
                 value(self.object)
             watcher = widget.param.watch(action, 'clicks')
+        elif kw_widget.get('throttled', False) and hasattr(widget, 'value_throttled'):
+            watcher = widget.param.watch(link_widget, 'value_throttled')
         else:
             watcher = widget.param.watch(link_widget, 'value')
         watchers.append(watcher)
@@ -462,6 +467,8 @@ class Param(PaneBase):
                 idx = self._callbacks.index(prev_watcher)
                 self._callbacks[idx] = watchers[0]
                 return
+            elif kw_widget.get('throttled', False) and hasattr(widget, 'value_throttled'):
+                updates['value_throttled'] = change.new
             else:
                 updates['value'] = change.new
 
