@@ -10,6 +10,7 @@ import uuid
 
 from collections import OrderedDict
 from functools import partial
+from urllib.parse import urljoin
 
 import param
 
@@ -141,12 +142,19 @@ class BaseTemplate(param.Parameterized, ServableMixin):
             loc = self._add_location(doc, location)
             doc.on_session_destroyed(loc._server_destroy)
         doc.title = title
+
+        # Initialize fake root. This is needed to ensure preprocessors
+        # which assume that all models are owned by a single root can
+        # link objects across multiple roots in a template.
         col = Column()
         preprocess_root = col.get_root(doc, comm)
+        col._hooks.append(self._apply_hooks)
         ref = preprocess_root.ref['id']
+
         for name, (obj, tags) in self._render_items.items():
             if self._apply_hooks not in obj._hooks:
                 obj._hooks.append(self._apply_hooks)
+            # We skip preprocessing on the individual roots
             model = obj.get_root(doc, comm, preprocess=False)
             mref = model.ref['id']
             doc.on_session_destroyed(obj._server_destroy)
@@ -164,9 +172,10 @@ class BaseTemplate(param.Parameterized, ServableMixin):
             self._apply_root(name, model, tags)
             add_to_doc(model, doc, hold=bool(comm))
 
+        # Here we ensure that the preprocessor is run across all roots
+        # and set up session cleanup hooks for the fake root.
         state._fake_roots.append(ref)
         state._views[ref] = (col, preprocess_root, doc, comm)
-
         col._preprocess(preprocess_root)
         col._documents[doc] = preprocess_root
         doc.on_session_destroyed(col._server_destroy)
@@ -447,7 +456,11 @@ class BasicTemplate(BaseTemplate):
     def _template_resources(self):
         name = type(self).__name__.lower()
         resources = _settings.resources(default="server")
-        dist_path = LOCAL_DIST if resources == 'server' else CDN_DIST
+        base_url = state.base_url[1:] if state.base_url.startswith('/') else state.base_url        
+        if resources == 'server':
+            dist_path = urljoin(base_url, LOCAL_DIST)
+        else:
+            dist_path = CDN_DIST
 
         # External resources
         css_files = dict(self._resources['css'])
